import ast


def get_annotation_string(annotation):
    annotation_map = {
        ast.Name: lambda a: a.id,
        ast.Subscript: lambda a: f"{get_annotation_string(a.value)}[{get_annotation_string(a.slice)}]",
        ast.Tuple: lambda a: f"({', '.join([get_annotation_string(elt) for elt in a.elts])})",
        ast.Attribute: lambda a: f"{get_annotation_string(a.value)}.{a.attr}",
        ast.List: lambda a: f"List[{get_annotation_string(a.elts[0]) if a.elts else 'Any'}]",
        ast.Dict: lambda a: f"Dict[{get_annotation_string(a.keys[0]) if a.keys else 'Any'}, {get_annotation_string(a.values[0]) if a.values else 'Any'}]",
        ast.Constant: lambda a: str(a.value)
    }
    return annotation_map.get(type(annotation), lambda a: "Any")(annotation)



import ast

from .annotation_string import get_annotation_string
from .helper import get_function_params
from .options import INCLUDE_ANY_TYPE


def process_node(node, parent_node, file_path, directory):
    if isinstance(node, ast.FunctionDef):
        process_function_node(node, parent_node, file_path, directory)
    elif isinstance(node, ast.ClassDef):
        process_class_node(node, parent_node, file_path, directory)


def process_function_node(node, parent_node, file_path, directory):
    name = node.name
    info = {
        "relative_path": str(file_path.relative_to(directory)),
        "type": "function",
        "params": get_function_params(node),
    }

    if node.returns:
        info["return_type"] = get_annotation_string(node.returns)
    elif INCLUDE_ANY_TYPE:
        info["return_type"] = "Any"

    handle_children(node, info, file_path, directory)

    parent_node[name] = info


def handle_children(node, info, file_path, directory):
    children_info = {}
    for sub_node in node.body:
        process_node(sub_node, children_info, file_path, directory)
    if len(children_info) > 0:
        info["children"] = children_info


def process_class_node(node, parent_node, file_path, directory):
    class_name = node.name
    info = {
        "relative_path": str(file_path.relative_to(directory)),
        "type": "class",
    }
    handle_children(node, info, file_path, directory)
    parent_node[class_name] = info



from .annotation_string import get_annotation_string
from .options import INCLUDE_ANY_TYPE


def get_function_params(node):
    function_params = []
    for arg in node.args.args:
        function_param = {"name": arg.arg }
        if arg.annotation:
            function_param["type"] = get_annotation_string(arg.annotation)
        elif INCLUDE_ANY_TYPE:
            function_param["type"] = "Any"

        function_params.append(function_param)

    return function_params



import ast
from pathlib import Path

from .nodes import process_node


def walk(directory: Path) -> dict:
    repository_info: dict = {}

    for file_path in directory.rglob("*.py"):
        relative_path = file_path.relative_to(directory)
        if ".venv" in str(relative_path):
            continue
        package_path = relative_path.parent

        root_node = repository_info
        for part in package_path.parts:
            root_node = root_node.setdefault(part, {"type": "package"})

        try:
            with file_path.open("r") as f:
                tree = ast.parse(f.read())

            for node in tree.body:
                process_node(node, root_node, file_path, directory)
        except SyntaxError as e:
            print(f"Skipping file {file_path} due to syntax error: {str(e)}")
            continue

    return repository_info